unit dnDBUtil;

(*****************************************************************************
Copyright  1997-99 Dieter Nth
version    : 2.1
date       : 10.08.1999
description:

  some internal Utility routines
******************************************************************************)

{$I dnInc.INC}

interface

uses
  Windows, BDE, SysUtils, Classes, DB, DBTables,
  dnUtil;

type
  TBDEData = record
    case TFieldType of
      ftString: (aString: array[0..255] of char);
      ftSmallint: (aSmallint: Smallint);
      ftInteger: (aInteger: Longint);
      ftWord: (aWord: Word);
      ftBoolean: (aBoolean: WordBool);
      ftFloat: (aFloat: Double);
      ftCurrency: (aDouble: Double);
      ftBCD: (aBCD: fmtBCD);
      ftTime: (aTime: Longint);
      ftDate: (aDate: Longint);
      ftDateTime: (aDateTime: TDateTime);
  end;

{*** Converting BDE-Data from/to Variants}
function BDEData2Variant(BDEData: pointer; //BDE internal data format
  Field: TField) //corresponding Field
  : variant;

procedure Variant2BDEData(value: variant;
  BDEData: pointer; //BDE internal data format
  Field: TField); //corresponding Field

{*** For Paradox-tables: returns the ValCheck defined for aField, FALSE if none exists.
     Extract values from VCD with BDEData2Variant}
function GetValCheck(aField: TField; var VCD: VChkDesc): boolean;

{*** Returns full path + filename for Table (Table must be active!!)}
function QualifiedTableName(Table: TTable): string;

{*** Restructure Table using OptParams:
     a String like 'LEVEL=7;FILL FACTOR=75;BLOCK SIZE=4096' OR
     a StringList with strings[0] := 'LEVEL=7'
                       strings[1] := 'FILL FACTOR=75'
                       strings[2] := 'BLOCK SIZE=4096'
     OptParams are Encoded via the EncodeOptData-functions below
     WARNING: All auxiliary passwords (but NOT the MASTER-PASSWORD) are removed}
procedure ChangeTableParams(Table: TTable; const OptParams: string);
procedure ChangeTableParamsList(Table: TTable; OptParams: TStringList);

{*** Create optional data for create/reconstruct of a BDE table using
     a String containing 'Name=value;Name=value;...' OR
     a StringList with one Param in every line}
function EncodeOptData(const Params: string;
  {String containing 'Name=value;Name=value;...'}
  var pOptDesc: pFldDesc;
  {must be freed with FreeMem(pOptDesc, OptCount * sizeof(FLDDesc))}
  var pOptData: pChar;
  {must be freed with strDispose}
  var OptCount: integer
  {count of Optional Data}
  ): boolean;

function EncodeOptDataList(Params: TStringList; {Strings containing 'Name=value'}
  var pOptDesc: pFldDesc;
  var pOptData: pChar;
  var OptCount: integer
  ): boolean;

implementation
uses
  DBCommon;

procedure Variant2BDEData(value: variant; BDEData: pointer; Field: TField);

  procedure Variant2DateTime(DataType: TFieldType);
  var
    ts: TTimeStamp;
  begin
    ts := DateTimeToTimeStamp(VarToDateTime(value));
    case DataType of
      ftTime: TBDEData(BDEData^).aTime := ts.Time;
      ftDate: TBDEData(BDEData^).aDate := ts.Date;
      ftDateTime: TBDEData(BDEData^).aDateTime := TimeStampToMSecs(ts);
    end;
  end;

begin
  if BDEData <> nil then
    with TBDEData(BDEData^) do
      case Field.DataType of
        ftSmallint: aSmallint := value;
        ftInteger: aInteger := value;
        ftWord: aWord := value;
        ftBoolean: aBoolean := value;
        ftFloat,
          ftCurrency: aFloat := value;
        ftBCD: CurrToFMTBCD(value, aBCD, 32, Field.Size);
        ftTime, ftDate, ftDateTime: Variant2DateTime(Field.DataType);
        ftString:
          begin
            StrPCopy(aString, value);
            if TStringField(Field).Transliterate then
              Field.DataSet.Translate(aString, aString, True);
          end;
      end;
end;

function BDEData2Variant(BDEData: pointer; Field: TField): variant;
var
  curr: currency;
  TimeStamp: TTimeStamp;
  s: string;
begin
  result := NULL;
  with TBDEData(BDEData^) do
    case Field.DataType of
      ftSmallint: result := aSmallint;
      ftInteger: result := aInteger;
      ftWord: result := aWord;
      ftBoolean: result := aBoolean;
      ftFloat,
        ftCurrency: result := aFloat;
      ftBCD:
        if FMTBCDToCurr(aBCD, curr) then result := curr;
      ftTime,
        ftDate,
        ftDateTime:
        begin
          with TBDEData(BDEData^) do
            case Field.DataType of
              ftDate:
                begin
                  TimeStamp.Time := 0;
                  TimeStamp.Date := aDate;
                end;
              ftTime:
                begin
                  TimeStamp.Time := aTime;
                  TimeStamp.Date := DateDelta;
                end;
            else
              try
                TimeStamp := MSecsToTimeStamp(aDateTime);
              except
                TimeStamp.Time := 0;
                TimeStamp.Date := 0;
              end;
            end;
          result := VarFromDateTime(TimeStampToDateTime(TimeStamp));
        end;
      ftString:
        begin
          s := StrPas(aString);
          if TStringField(Field).Transliterate and (s <> '') then
            Field.DataSet.Translate(pChar(s), pChar(s), False);
          result := s;
        end;
    end;
end;

function GetValCheck(aField: TField; var VCD: VChkDesc): boolean;
var
  CursorProps: CurProps;
  i: integer;
begin
  result := True;
  with TTable(aField.DataSet) do
  begin
    DbiGetCursorProps(Handle, CursorProps);
    for i := 1 to CursorProps.iValChecks do
    begin
      DbiGetVChkDesc(Handle, i, @VCD);
      if VCD.iFldNum = aField.FieldNo then exit;
    end;
  end;
  FillChar(VCD, sizeOf(VCD), #0);
  result := False;
end;

function QualifiedTableName(Table: TTable): string;
var
  Props: CurProps;
begin
  result := '';
  try
    if DbiGetCursorProps(Table.Handle, Props) <> DBIErr_None then exit;
    SetLength(result, DBIMAXPATHLEN);
    if DbiFormFullName(Table.DBHandle, pChar(Table.TableName),
      Props.szTableType, pChar(result)) <> DBIErr_None then result := '';
  except
  end;
end;

function EncodeOptData(const Params: string;
  var pOptDesc: pFldDesc;
  var pOptData: pChar;
  var OptCount: integer
  ): boolean;
var
  i: integer;
  sl: TStringList;
begin
  sl := TStringList.create;
  try
    i := 1;
    while i <= Length(Params) do
      sl.Add(ExtractFieldName(Params, i));
    result := EncodeOptDataList(sl, pOptDesc, pOptData, OptCount);
  finally
    sl.free;
  end;
end;

function EncodeOptDataList(Params: TStringList;
  var pOptDesc: pFldDesc;
  var pOptData: pChar;
  var OptCount: integer
  ): boolean;
var
  DataSize,
    i: integer;
  newOptDesc: pFldDesc;
  newOptData: pChar;

  procedure CalcOptDataSize;
  var
    i, len, Size: integer;
  begin
    DataSize := 0;
    OptCount := 0;
    with Params do
    begin
      for i := 0 to Count - 1 do
      begin
        len := Length(Strings[i]);
        Size := succ(len - Pos('=', Strings[i]));
        if (Size < len) and (Size > 1) then
        begin
          inc(OptCount);
          inc(DataSize, Size);
        end;
      end;
    end;
  end;

  procedure EncodeOptString(const Data: string);
  var
    p: integer;
  begin
    p := Pos('=', Data);
    if (p > 1) and (p < Length(Data)) then
      with newOptDesc^ do
      begin
        iLen := Length(Data) - p + 1;
        StrUpper(StrPLCopy(szName, Data, p - 1));
        StrLCopy(newOptData, @Data[p + 1], Length(Data) - p);
        iOffset := newOptData - pOptData;
        inc(newOptData, iLen);
        inc(newOptDesc);
      end;
  end;

begin
  result := False;
  newOptData := nil;
  newOptDesc := nil;
  CalcOptDataSize;
  if OptCount > 0 then
    with Params do
    try
      newOptDesc := AllocMem(OptCount * sizeOf(FLDDesc));
      newOptData := StrAlloc(DataSize);
      pOptData := newOptData;
      pOptDesc := newOptDesc;
      for i := 0 to Count - 1 do
        EncodeOptString(Strings[i]);
      result := True;
    except
      if assigned(newOptData) then StrDispose(newOptData);
      if assigned(newOptDesc) then
        FreeMem(newOptDesc, OptCount * sizeOf(FLDDesc));
      pOptData := nil;
      pOptDesc := nil;
      raise;
    end;
end;

procedure ChangeTableParamsList(Table: TTable; OptParams: TStringList);
var
  hDB: hDBIDb;
  TableDesc: CRTblDesc;
  wasExclusive,
    wasActive: boolean;
  OptDesc: pFldDesc;
  OptData: pChar;
  OptCount: integer;
  x: Word;
begin
  with Table do
  begin
    wasActive := active;
    wasExclusive := Exclusive;
    DisableControls;
    try
      active := False;
      Exclusive := True;
      active := True;
      if EncodeOptDataList(OptParams,
        OptDesc, OptData, OptCount) then
      try
        with TableDesc do
        try
          FillChar(TableDesc, sizeOf(TableDesc), #0);
          AnsiToNative(Locale, TableName, szTblName, sizeOf(szTblName) - 1);
          Check(DbiGetProp(hDBIObj(Handle), drvDRIVERTYPE, @szTblType,
            DBIMAXNAMELEN, x));
          hDB := DBHandle;
          active := False;
          iOptParams := OptCount;
          pFldOptParams := OptDesc;
          pOptData := OptData;
          bProtected := True;
          x := DBIDoRestructure(hDB, 1, @TableDesc, nil, nil, nil, False);
          if x = DBIERR_INVALIDPASSWORD then
          begin
            bProtected := False;
            x := DBIDoRestructure(hDB, 1, @TableDesc, nil, nil, nil, False);
          end;
          Check(x);
        finally
          StrDispose(OptData);
          FreeMem(OptDesc, OptCount * sizeOf(FLDDesc));
        end;
      finally
        Exclusive := wasExclusive;
        active := wasActive;
      end;
    finally
      EnableControls;
    end;
  end;
end;

procedure ChangeTableParams(Table: TTable; const OptParams: string);
var
  i: integer;
  sl: TStringList;
begin
  sl := TStringList.create;
  try
    i := 1;
    while i <= Length(OptParams) do
      sl.Add(ExtractFieldName(OptParams, i));
    ChangeTableParamsList(Table, sl);
  finally
    sl.free;
  end;
end;

end.
