unit dnPxFlds;

(*****************************************************************************
Copyright  1997-99 Dieter Nth
version    : 2.1
date       : 10.08.1999
description:

  Enhanced TFields with Paradox-mask support.
  All those Fields have ONE new property: PxMask

  Other props Px_ErrorMessage, Px_Mask, Px_AutoValidate, Px_AutoFill,
  Px_RaiseInputError are ONLY defined to support Borland's DataDictionary.
  So, if you don't want to use the DataDictionary UNDEFINE UseDD in dnInc.INC

******************************************************************************)

interface

{$I dnInc.INC}

uses
  Windows, BDE, SysUtils, Classes, DB, DBTables, Dialogs,
  dnPxMask, dnGlobal;

{*** Load/Store PxMask-properties from/to an IniFile; if ClearStoredFlag is set
     (this is not yet supported), then those PxMasks will not be stored in
     DFM-file (in design-mode), i.e. they will be LOST next time you open the
     form, so you'll have to call LoadPxMasksFromIni in DataSet's OnAfterOpen.
     If DataSet is a non-SQL Table and PXM-file is in the same directory
     as Table then e.g.
       procedure TForm1.Table1AfterOpen(DataSet: TDataSet);
       begin
         LoadPxMasksFromIni(DataSet,
           ChangeFileExt(QualifiedTableName(DataSet as TTable), PxMaskFileExt),
           true); //that's how TPxTable does it automatically
       end;

     function QualifiedTableName defined in dnDBUtil.pas
     const PxMaskFileExt defined in dnGlobal.pas
***}

function LoadPxMasksFromIni( DataSet: TDataSet;
                             const IniFileName : string;
                             ClearStoredFlag: boolean): boolean;

function StorePxMasksToIni( DataSet: TDataSet;
                            const IniFileName : string;
                            ClearStoredFlag: boolean): boolean;

type
{ TPxStringField }
  TPxStringField = class(TStringField)
  protected
    function GetAsString: string; override;
  {$I dnPxFlds.T1}

{ TPxDateTimeField }
  TPxDateTimeField = class(TDateTimeField)
  {$I dnPxFlds.T1}

{ TPxDateField }
  TPxDateField = class(TDateField)
  {$I dnPxFlds.T1}

{ TPxTimeField }
  TPxTimeField = class(TTimeField)
  {$I dnPxFlds.T1}

{ TPxIntegerField }
  TPxIntegerField = class(TIntegerField)
  {$I dnPxFlds.T1}

{ TPxSmallintField }
  TPxSmallIntField = class(TSmallIntField)
  {$I dnPxFlds.T1}

{ TPxWordField }
  TPxWordField = class(TWordField)
  {$I dnPxFlds.T1}

{ TPxFloatField }
  TPxFloatField = class(TFloatField)
  {$I dnPxFlds.T1}

{ TPxCurrencyField }
  TPxCurrencyField = class(TCurrencyField)
  {$I dnPxFlds.T1}

{ TPxBCDField }
  TPxBCDField = class(TBCDField)
  {$I dnPxFlds.T1}

implementation
uses
  DBConsts, TypInfo,
  dnPxCnst, dnUtil, dnDBUtil, IniFiles;

function TPxStringField.GetAsString: string;
begin
  Result := inherited GetAsString;
  with FPxMask do
    if (result <> '') and (length(result) = Size) and AutoFill then
      PxMaskCheckInput(Mask, result, true);
end;

{$DEFINE PxStringField}
  {$I dnPxFlds.T2}
{$UNDEF PxStringField}

{$DEFINE PxDateTimeField}
  {$I dnPxFlds.T2}
{$UNDEF PxDateTimeField}

{$DEFINE PxDateField }
  {$I dnPxFlds.T2}
{$UNDEF PxDateField }

{$DEFINE PxTimeField }
  {$I dnPxFlds.T2}
{$UNDEF PxTimeField }

{$DEFINE PxIntegerField }
  {$I dnPxFlds.T2}
{$UNDEF PxIntegerField }

{$DEFINE PxSmallintField }
  {$I dnPxFlds.T2}
{$UNDEF PxSmallintField }

{$DEFINE PxWordField }
  {$I dnPxFlds.T2}
{$UNDEF PxWordField }

{$DEFINE PxFloatField }
  {$I dnPxFlds.T2}
{$UNDEF PxFloatField }

{$DEFINE PxCurrencyField }
  {$I dnPxFlds.T2}
{$UNDEF PxCurrencyField }

{$DEFINE PxBCDField }
  {$I dnPxFlds.T2}
{$UNDEF PxBCDField }

const
 sMask = 'Mask';
 sErrorMessage = 'ErrorMessage';
 sAutoValidate = 'AutoValidate';
 sAutoFill = 'AutoFill';
 sRaiseInputError = 'RaiseInputError';

function LoadPxMasksFromIni(DataSet: TDataSet; const IniFileName : string;
                               ClearStoredFlag: boolean): boolean;
var
  IniFile : TIniFile;
  List: TStringList;
  pxm: TPxMask;
  i : integer;
  s : string;
begin
  result := false;
  if IniFileName = '' then exit;
  IniFile := TIniFile.create(IniFileName);
  try
    List:= TStringList.create;
    with DataSet do
    try
      IniFile.ReadSections(List);
      for i := 0 to List.count - 1 do
      begin
        s := List[i];
        if HasPxMask( FindField(s), pxm) then
        with pxm do begin
          Mask := IniFile.ReadString(s, sMask, '');
          ErrorMessage := IniFile.ReadString(s, sErrorMessage, '');
          AutoValidate := IniFile.ReadBool(s, sAutoValidate, AutoValidateDefault);
          AutoFill := IniFile.ReadBool(s, sAutoFill, AutoFillDefault);
          RaiseInputError := IniFile.ReadBool(s, sRaiseInputError, RaiseInputErrorDefault);
          if ClearStoredFlag then exclude(FState, pxStored);
        end;
      end;
      result := true;
    finally
      List.free;
    end;
  finally
    IniFile.free;
  end;
end;

function StorePxMasksToIni(DataSet: TDataSet; const IniFileName : string;
                               ClearStoredFlag: boolean): boolean;
var
  IniFile : TIniFile;
  i : integer;
  s : string;
  pxm : TPxMask;
begin
  result := false;
  if IniFileName = '' then exit;
  IniFile := TIniFile.create(IniFileName);
  with DataSet do try
    for i := 0 to FieldCount - 1 do
      if HasPxMask(Fields[i], pxm) then
      with pxm do begin
        s := Fields[i].FieldName;
        IniFile.EraseSection(s);
        if (Mask <> '') then
          IniFile.WriteString(s, sMask, Mask);
        if ErrorMessage <> '' then
          IniFile.WriteString(s, sErrorMessage, ErrorMessage);
        if AutoValidate <> AutoValidateDefault then
          IniFile.WriteBool(s, sAutoValidate, not AutoValidateDefault);
        if AutoFill <> AutoFillDefault then
          IniFile.WriteBool(s, sAutoFill, not AutoFillDefault);
        if RaiseInputError  <> RaiseInputErrorDefault then
          IniFile.WriteBool(s, sRaiseInputError, not RaiseInputErrorDefault);
        if ClearStoredFlag then exclude(FState, pxStored);
      end;
  finally
    IniFile.free;
  end;
end;

procedure ModifyFieldClasses;
begin
  DefaultFieldClasses[ftString] := TPxStringField;
  DefaultFieldClasses[ftDateTime] := TPxDateTimeField;
  DefaultFieldClasses[ftDate] := TPxDateField;
  DefaultFieldClasses[ftTime] := TPxTimeField;
  DefaultFieldClasses[ftInteger] := TPxIntegerField;
  DefaultFieldClasses[ftSmallint] := TPxSmallintField;
  DefaultFieldClasses[ftWord] := TPxWordField;
  DefaultFieldClasses[ftFloat] := TPxFloatField;
  DefaultFieldClasses[ftBCD] := TPxBCDField;
  DefaultFieldClasses[ftCurrency] := TPxCurrencyField;
end;

begin
{$IFDEF ModifyFieldClasses}
  ModifyFieldClasses;
{$ENDIF}
end.

